"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.deployExecutor = deployExecutor;
const tslib_1 = require("tslib");
const devkit_1 = require("@nx/devkit");
const core_1 = require("@nx-extend/core");
const path_1 = require("path");
const get_valid_secrets_1 = require("../../utils/get-valid-secrets");
function deployExecutor(options, context) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        var _a, _b;
        const { functionName, region, trigger = 'http', triggerValue = null, triggerEvent = null, triggerLocation = null, envVarsFile = null, envVars, maxInstances = 10, project = null, memory = '128MB', serviceAccount = null, entryPoint = null, retry = false, egressSettings = null, vpcConnector = null, securityLevel = null, secrets = [], gen = 2, concurrency = 1, cloudSqlInstance = null, timeout = null, cpu = 1 } = options;
        let runtime = options.runtime || 'nodejs20';
        // If "recommended" option is selected set the currently recommended one of Google (https://cloud.google.com/functions/docs/concepts/nodejs-runtime)
        if (runtime === 'recommended') {
            runtime = 'nodejs20';
        }
        // Options with default values based of trigger type
        const { ingressSettings = trigger === 'http' ? null : 'internal-only', allowUnauthenticated = trigger === 'http' } = options;
        let correctMemory = memory;
        let correctCPU = cpu;
        if (gen === 2) {
            // Use the correct memory for the right gen
            if (memory.endsWith('MB')) {
                correctMemory = memory.replace('MB', 'Mi');
            }
            if (concurrency > 1 && correctCPU < 1) {
                devkit_1.logger.warn('Setting "cpu" to "1" because concurrency is higher the 1');
                correctCPU = 1;
            }
        }
        if (triggerValue && trigger === 'http') {
            throw new Error('"triggerValue" is not accepted when trigger is "http"!');
        }
        const { targets } = context.workspace.projects[context.projectName];
        const validSecrets = (0, get_valid_secrets_1.getValidSecrets)(secrets);
        const setEnvVars = Object.keys(envVars || {}).reduce((env, envVar) => {
            env.push(`${envVar}=${envVars[envVar]}`);
            return env;
        }, []);
        let { success } = (0, core_1.execCommand)((0, core_1.buildCommand)([
            `gcloud functions deploy`,
            functionName,
            gen === 2 && '--gen2',
            `--trigger-${trigger}${triggerValue ? `=${triggerValue}` : ''}`,
            triggerEvent && `--trigger-event=${triggerEvent}`,
            triggerLocation && `--trigger-location=${triggerLocation}`,
            `--runtime=${runtime}`,
            `--memory=${correctMemory}`,
            `--region=${region}`,
            concurrency > 0 && `--concurrency=${concurrency}`,
            correctCPU && `--cpu=${correctCPU}`,
            entryPoint && `--entry-point=${entryPoint}`,
            envVarsFile && `--env-vars-file=${envVarsFile}`,
            setEnvVars.length > 0 && `--set-env-vars=${setEnvVars.join(',')}`,
            retry && `--retry`,
            ingressSettings && `--ingress-settings=${ingressSettings}`,
            egressSettings && `--egress-settings=${egressSettings}`,
            vpcConnector && `--vpc-connector=${vpcConnector}`,
            securityLevel && `--security-level=${securityLevel}`,
            timeout && `--timeout=${timeout}`,
            `--source=${(0, path_1.join)(context.root, (_b = (_a = targets === null || targets === void 0 ? void 0 : targets.build) === null || _a === void 0 ? void 0 : _a.options) === null || _b === void 0 ? void 0 : _b.outputPath.toString())}`,
            `--max-instances=${maxInstances}`,
            allowUnauthenticated && '--allow-unauthenticated',
            serviceAccount && `--service-account=${serviceAccount}`,
            validSecrets.length > 0 && `--set-secrets=${validSecrets.join(',')}`,
            project && `--project=${project}`,
            '--quiet'
        ]));
        if (success &&
            gen === 2 &&
            cloudSqlInstance) {
            devkit_1.logger.info('Updating service with more configurations');
            const serviceUpdateCommand = (0, core_1.execCommand)((0, core_1.buildCommand)([
                'gcloud run services update',
                functionName,
                cloudSqlInstance && `--add-cloudsql-instances=${cloudSqlInstance}`,
                `--region=${region}`,
                project && `--project=${project}`,
                '--quiet'
            ]));
            success = serviceUpdateCommand.success;
        }
        return Promise.resolve({ success });
    });
}
exports.default = deployExecutor;
//# sourceMappingURL=deploy.impl.js.map